==> hermes-5.6.0.23.tar.gz: hermes-5.6.0.23/app/models/hermes/threshold.rb <==
# frozen_string_literal: true

module Hermes
  # Threshold clas
  class Threshold
    include Hermes::Concerns::CommonFieldsConcern
    field :type, type: String
    field :codes, type: Array, default: [] 
    field :thresholds_map, type: Hash

    has_many :housing_offers, class_name: 'Hermes::HousingOffer'

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    after_save :after_actions

    common_indexes *%i[
      type
      codes
    ]

    LABEL_MAPPING =  {
      Z1: "Paris et communes limitrophes",
      Z2: "IDF hors Paris et Communes limitrophes",
      Z3: "Autres régions",
      Z4: "Autres régions 170%",
      Z5: "Autres régions 150%",
      Z6: "Paris",
      Z7: "Zone centrale IDF",
      Z8: "Reste IDF",
      ZA: "Zone A",
      ZB: "Zone B",
      ZC: "Zone C",
      ZAB: "Zone A Bis",
      ZB1: "Zone B1",
      ZB2: "Zone B2",
      I01:	"PLI03/01 Paris & Commun",
      I02:	"PLI03/01 IDF H Paris&com",
      I03:	"PLI03/01 Autres régions",
      I04:	"PLI 8/2004 Zone A",
      I05:	"PLI 8/2004 Zone B",
      I06:	"PLI 8/2004 Zone C",
      I07:	"PLI 2015 Zone Abis",
      I08:	"PLI 2015 Zone A",
      I09:	"PLI 2015 Zone B1",
      I10:	"PLI 2015 Zone B2",
      I11:	"PLI 2015 Zone C",
    }

    validates_presence_of :type, :thresholds_map
    #TODO : handle string data for thresholds_map
    def subtype(subtype_name)
      thresholds_map[subtype_name]
    end

    def thresholds_map=(h)
      super(
        if h.is_a? Hash
          h.all_to(Integer)
        else
          {}
        end
      )
    end

    def self.label_mapping
      LABEL_MAPPING
    end 

    def thresholds(subtype_name, type_name)
      sub_type = subtype(subtype_name)
      return unless sub_type

      field_name = case type_name
                   when 'ceils'
                     'thresholds'
                   else
                     type_name
                   end
      sub_type["#{field_name}_list"]
    end

    def threshold(subtype_name, type_name, index)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      thresholds[index].to_i || 0.0
    end

    def threshold_addition(subtype_name, type_name, index)
      sub_type = subtype(subtype_name)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      addition_value = sub_type['another_person']
      return unless addition_value

      thresholds.last.to_i + (addition_value.to_i * index) || 0.0
    end

    def set_offer_thresholds(offer)
      return unless offer.threshold_subtype
      offer_subtype = LABEL_MAPPING[offer.threshold_subtype.to_sym] || offer.threshold_subtype

      %w[ceils floors].each do |field_type|
        (1..6).each do |i|
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold(offer_subtype, field_type, i - 1) || 0.0
          )
        end
        (7..12).each do |i|
          is_zero = offer["#{field_type.singularize}_category_6"] == 0.0
          j = i - 6
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            is_zero ? 0.0 : threshold_addition(offer_subtype, field_type, j) || 0.0
          )
        end
      end
    end

    def after_actions
      UpdateThresholdOffersJob.perform_later(self.id.to_s)
    end
  end
end


# mapping = {
#   'PLA I'=> ['PLN'],
#   'PLA LM'=> ['PLM'],
#   'PLS 7/92 à 8/93'=> ['PS2'],
#   'PLI 3/96 à 2/01'=> ['PL3'],
#   'PLA'=> ['PLA'],
#   'PLS 1/97 (sans dérogation)'=> ['PS1'],
#   'PLI AVANT 7/92'=> ['PL1'],
#   'PLI 8/93 à 2/96'=> ['PL2'],
#   'PLI 8/2004 à 12/2014'=> ['PL5'],
#   'PLI 2015'=> ['PL6'],
#   'PLA TS'=> ['PTS'],
#   'HLM'=> ['HLM', 'HLO'],
#   'PLUS'=> ['PLU'],
#   'PLA CCF (sans dérogation)'=> ['CFF'],
#   'LLI'=> ['LLI'],
#   'ILM (SA HLM)'=> ['IL1'],
#   'PINEL'=> ['PIN'],
#   'BESSON NEUF'=> ['BNE', 'BOR'],
#   'BESSON ANCIEN'=> ['BAN'],
# }

# mapping.each do |k, c|
#   t = Hermes::Threshold.find_by(type: k)
#   # puts t.type
#   t.codes = c
#   t.save
# end

# Hermes::Threshold.all().pluck(:type)

# t = Hermes::Threshold.create(
#   type: "LLS DOM", 
#   codes: ["LLD"], 
#   thresholds_map: {
#     "LLS DOM"=>{"thresholds_list"=>[18561, 24786, 29807, 35984, 42332, 47707], "another_person"=>5321}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "LLTS DOM", 
#   codes: ["LLT"], 
#   thresholds_map: {
#     "LLTS DOM"=>{"thresholds_list"=>[13921, 18590, 22355, 26988, 31749, 35780], "another_person"=>3991}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "PLS DOM", 
#   codes: ["PS4"], 
#   thresholds_map: {
#     "PLS DOM"=>{"thresholds_list"=>[24129, 32222, 38749, 38749, 55032, 62019], "another_person"=>3991}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "60% du PLUS", 
#   codes: ["PMI"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[14233, 21271, 27884, 33292, 39610, 44572], "another_person"=>4967}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[14233, 21271, 25570, 30628, 36257, 36257], "another_person"=>4546}, 
#     "Autres régions"=>{"thresholds_list"=>[12374, 16524, 19871, 23989, 28221, 31805], "another_person"=>3547}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "130% du PLUS", 
#   codes: ["PCL", "PS3", "FON"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{
#       "thresholds_list"=>[30837, 46088, 60415, 72132, 85822, 96572], 
#       "floors_list"=>[23721, 35452, 46473, 55486, 66017, 74286], 
#       "another_person"=>10761
#     }, 
#     "IDF hors Paris et Communes limitrophes"=>{
#       "thresholds_list"=>[30837, 46088, 55401, 66360, 78558, 88401], 
#       "floors_list"=>[23721, 35452, 42616, 51046, 60429, 68001], 
#       "another_person"=>9850
#     }, 
#     "Autres régions"=>{
#       "thresholds_list"=>[26810, 35802, 43055, 51977, 61146, 68910], 
#       "floors_list"=>[20623, 27540, 33119, 39982, 47035, 53008], 
#       "another_person"=>7686
#     }, 
#   }
# )


# t = Hermes::Threshold.create(
#   type: "120% du PLUS", 
#   codes: ["PMA"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[28465, 42542, 55768, 66583, 79220, 89143], "another_person"=>9934}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[28465, 42542, 51139, 61255, 72515, 81601], "another_person"=>9092}, 
#     "Autres régions"=>{"thresholds_list"=>[24748, 33048, 39743, 47978, 56442, 56442], "another_person"=>7094}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "150% du PLUS", 
#   codes: ["IL2"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[35582, 53178, 69710, 69710, 99026, 111429], "another_person"=>12417}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[35582, 53178, 63924, 76569, 90644, 102002], "another_person"=>11366}, 
#     "Autres régions"=>{"thresholds_list"=>[30935, 41310, 49679, 59973, 70553, 70553], "another_person"=>8868}, 
#   }
# )

==> hermes-5.6.0.33.tar.gz: hermes-5.6.0.33/app/models/hermes/threshold.rb <==
# frozen_string_literal: true

module Hermes
  # Threshold clas
  class Threshold
    include Hermes::Concerns::CommonFieldsConcern
    field :type, type: String
    field :codes, type: Array, default: [] 
    field :thresholds_map, type: Hash

    has_many :housing_offers, class_name: 'Hermes::HousingOffer'

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    after_save :after_actions

    common_indexes *%i[
      type
      codes
    ]

    LABEL_MAPPING =  {
      Z1: "Paris et communes limitrophes",
      Z2: "IDF hors Paris et Communes limitrophes",
      Z3: "Autres régions",
      Z4: "Autres régions 170%",
      Z5: "Autres régions 150%",
      Z6: "Paris",
      Z7: "Zone centrale IDF",
      Z8: "Reste IDF",
      ZA: "Zone A",
      ZB: "Zone B",
      ZC: "Zone C",
      ZAB: "Zone A Bis",
      ZB1: "Zone B1",
      ZB2: "Zone B2",
      I01:	"PLI03/01 Paris & Commun",
      I02:	"PLI03/01 IDF H Paris&com",
      I03:	"PLI03/01 Autres régions",
      I04:	"PLI 8/2004 Zone A",
      I05:	"PLI 8/2004 Zone B",
      I06:	"PLI 8/2004 Zone C",
      I07:	"PLI 2015 Zone Abis",
      I08:	"PLI 2015 Zone A",
      I09:	"PLI 2015 Zone B1",
      I10:	"PLI 2015 Zone B2",
      I11:	"PLI 2015 Zone C",
    }

    validates_presence_of :type, :thresholds_map
    #TODO : handle string data for thresholds_map
    def subtype(subtype_name)
      thresholds_map[subtype_name]
    end

    def thresholds_map=(h)
      super(
        if h.is_a? Hash
          h.all_to(Integer)
        else
          {}
        end
      )
    end

    def self.label_mapping
      LABEL_MAPPING
    end 

    def thresholds(subtype_name, type_name)
      sub_type = subtype(subtype_name)
      return unless sub_type

      field_name = case type_name
                   when 'ceils'
                     'thresholds'
                   else
                     type_name
                   end
      sub_type["#{field_name}_list"]
    end

    def threshold(subtype_name, type_name, index)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      thresholds[index].to_i || 0.0
    end

    def threshold_addition(subtype_name, type_name, index)
      sub_type = subtype(subtype_name)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      addition_value = sub_type['another_person']
      return unless addition_value

      thresholds.last.to_i + (addition_value.to_i * index) || 0.0
    end

    def set_offer_thresholds(offer)
      return unless offer.threshold_subtype
      offer_subtype = LABEL_MAPPING[offer.threshold_subtype.to_sym] || offer.threshold_subtype

      %w[ceils floors].each do |field_type|
        (1..6).each do |i|
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold(offer_subtype, field_type, i - 1) || 0.0
          )
        end
        (7..12).each do |i|
          is_zero = offer["#{field_type.singularize}_category_6"] == 0.0
          j = i - 6
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            is_zero ? 0.0 : threshold_addition(offer_subtype, field_type, j) || 0.0
          )
        end
      end
    end

    def after_actions
      UpdateThresholdOffersJob.perform_later(
        self.id.to_s, 
        [type, type_was] + (codes || []) + (codes_was || [])
      )
    end
  end
end


# mapping = {
#   'PLA I'=> ['PLN'],
#   'PLA LM'=> ['PLM'],
#   'PLS 7/92 à 8/93'=> ['PS2'],
#   'PLI 3/96 à 2/01'=> ['PL3'],
#   'PLA'=> ['PLA'],
#   'PLS 1/97 (sans dérogation)'=> ['PS1'],
#   'PLI AVANT 7/92'=> ['PL1'],
#   'PLI 8/93 à 2/96'=> ['PL2'],
#   'PLI 8/2004 à 12/2014'=> ['PL5'],
#   'PLI 2015'=> ['PL6'],
#   'PLA TS'=> ['PTS'],
#   'HLM'=> ['HLM', 'HLO'],
#   'PLUS'=> ['PLU'],
#   'PLA CCF (sans dérogation)'=> ['CFF'],
#   'LLI'=> ['LLI'],
#   'ILM (SA HLM)'=> ['IL1'],
#   'PINEL'=> ['PIN'],
#   'BESSON NEUF'=> ['BNE', 'BOR'],
#   'BESSON ANCIEN'=> ['BAN'],
# }

# mapping.each do |k, c|
#   t = Hermes::Threshold.find_by(type: k)
#   # puts t.type
#   t.codes = c
#   t.save
# end

# Hermes::Threshold.all().pluck(:type)

# t = Hermes::Threshold.create(
#   type: "LLS DOM", 
#   codes: ["LLD"], 
#   thresholds_map: {
#     "LLS DOM"=>{"thresholds_list"=>[18561, 24786, 29807, 35984, 42332, 47707], "another_person"=>5321}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "LLTS DOM", 
#   codes: ["LLT"], 
#   thresholds_map: {
#     "LLTS DOM"=>{"thresholds_list"=>[13921, 18590, 22355, 26988, 31749, 35780], "another_person"=>3991}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "PLS DOM", 
#   codes: ["PS4"], 
#   thresholds_map: {
#     "PLS DOM"=>{"thresholds_list"=>[24129, 32222, 38749, 38749, 55032, 62019], "another_person"=>3991}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "60% du PLUS", 
#   codes: ["PMI"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[14233, 21271, 27884, 33292, 39610, 44572], "another_person"=>4967}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[14233, 21271, 25570, 30628, 36257, 36257], "another_person"=>4546}, 
#     "Autres régions"=>{"thresholds_list"=>[12374, 16524, 19871, 23989, 28221, 31805], "another_person"=>3547}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "130% du PLUS", 
#   codes: ["PCL", "PS3", "FON"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{
#       "thresholds_list"=>[30837, 46088, 60415, 72132, 85822, 96572], 
#       "floors_list"=>[23721, 35452, 46473, 55486, 66017, 74286], 
#       "another_person"=>10761
#     }, 
#     "IDF hors Paris et Communes limitrophes"=>{
#       "thresholds_list"=>[30837, 46088, 55401, 66360, 78558, 88401], 
#       "floors_list"=>[23721, 35452, 42616, 51046, 60429, 68001], 
#       "another_person"=>9850
#     }, 
#     "Autres régions"=>{
#       "thresholds_list"=>[26810, 35802, 43055, 51977, 61146, 68910], 
#       "floors_list"=>[20623, 27540, 33119, 39982, 47035, 53008], 
#       "another_person"=>7686
#     }, 
#   }
# )


# t = Hermes::Threshold.create(
#   type: "120% du PLUS", 
#   codes: ["PMA"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[28465, 42542, 55768, 66583, 79220, 89143], "another_person"=>9934}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[28465, 42542, 51139, 61255, 72515, 81601], "another_person"=>9092}, 
#     "Autres régions"=>{"thresholds_list"=>[24748, 33048, 39743, 47978, 56442, 56442], "another_person"=>7094}, 
#   }
# )

# t = Hermes::Threshold.create(
#   type: "150% du PLUS", 
#   codes: ["IL2"], 
#   thresholds_map: {
#     "Paris et communes limitrophes"=>{"thresholds_list"=>[35582, 53178, 69710, 69710, 99026, 111429], "another_person"=>12417}, 
#     "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[35582, 53178, 63924, 76569, 90644, 102002], "another_person"=>11366}, 
#     "Autres régions"=>{"thresholds_list"=>[30935, 41310, 49679, 59973, 70553, 70553], "another_person"=>8868}, 
#   }
# )
